%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Vasco M Carvalho
% From Micro to Macro via Production Networks
% Journal of Economic Perspectives, 2014
% Section 3 Computations and Figures 3 and 4
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;

%% LOAD DATA AND CREATE UNWEIGHTED VERSION OF DATA 
%Load 2002 Input Output Data (Commodity by Commodity Matrix)
load CC02.mat

%Discretize IO into binary matrix (Create Unweighted Directed Graph)
%Pick threshold rule: >1% of total intermediate input bill
t=0.01;

for i=1:max(size(CC02))
    for j=1:max(size(CC02))
        if CC02(i,j)>t
            io02(i,j)=1; 
        else
            io02(i,j)=0;
        end
    end
end


%% NETWORK DENSITY
net02=io02;
density=nnz(net02)/(max(size(net02))^2)


%% DISTANCES AND DIAMETERS
D02=distance_bin(net02);
[lambda,efficiency,ecc,radius,diameter] = charpath(D02);
distance=lambda
diameter


%% WEIGHTED OUTDEGREE DISTRIBUTION
% Define weighted outdegree distribution
odw=sum(CC02');

% Power law estimation
[alpha_MLE_02, xmin_02]=plfit(sum(CC02'),'nosmall');

%% CENTRALITY COMPUTATIONS
% Transforming share matrix so that all IO columns sum to 1 (following Acemoglu et
% al 2012, ECMA)
X=CC02;
for i=1:max(size(X))
    if sum(X(:,i))>0
        XX(:,i)=X(:,i)./sum(X(:,i));
    elseif sum(X(:,i))==0
        XX(:,i)=X(:,i);
    end
end
W=XX;
clear X XX;

% Compusting Centrality Score
v=(0.5/417)*inv(eye(417)-0.5*W);
centrality=sum(v');

% Power law estimation
[alpha_MLE_v02, xmin_v02]=plfit(sum(v'),'finite');


%% FIGURES 3 AND 4
% Figure 3: Weighted Outdegree Distribution
plplot(sum(CC02'),xmin_02,alpha_MLE_02);
axis([0.0001 100 0 1])
xlabel('Weighted Outdegree');
ylabel('Empirical CCDF');
curtick = get(gca, 'XTick');
set(gca, 'XTickLabel', cellstr(num2str(curtick(:))));
curtick = get(gca, 'YTick');
set(gca, 'YTickLabel', cellstr(num2str(curtick(:))));
axis square;

% Figure 4: Centrality Distribution
plplot(sum(v'),xmin_v02,alpha_MLE_v02);
axis([0.001 0.1 0 1])
xlabel('Centrality');
ylabel('Empirical CCDF');
curtick = get(gca, 'XTick');
set(gca, 'XTickLabel', cellstr(num2str(curtick(:))));
curtick = get(gca, 'YTick');
set(gca, 'YTickLabel', cellstr(num2str(curtick(:))));
axis square

%% SAVE WEIGHTED OUTDEGREE DISTRIBUTION AND CENTRALITY

csvwrite('weighted_outdegree_02.csv', odw')
csvwrite('centrality_02.csv', centrality')



