%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Vasco M Carvalho
% From Micro to Macro via Production Networks
% Journal of Economic Perspectives, 2014
% Section 4 Computations and Figures 5 and 6
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% IMPORT AND PREPARE NBER MANUFACTURING DATABASE

load nber_manufacturing_database.mat

% Assemble all data into Year by Sector Matrix
[index1]=find(sic==2011);

VADD=zeros(size(index1,1),459);
VSHIP=zeros(size(index1,1),459);
DTFP5=zeros(size(index1,1),459);
PSHIP=zeros(size(index1,1),459);
RVADD=zeros(size(index1,1),459);

VADD(:,1)=vadd(1:size(index1,1));
VSHIP(:,1)=vship(1:size(index1,1));
DTFP5(:,1)=dtfp5(1:size(index1,1));
PSHIP(:,1)=piship(1:size(index1,1));
RVADD(:,1)=vadd(1:size(index1,1))./piship(1:size(index1,1));
RSHIP(:,1)=vship(1:size(index1,1))./piship(1:size(index1,1));

for i=2:459
    counter=52*(i-1)+1;
    VADD(:,i)=vadd(counter:counter+51,1);
    VSHIP(:,i)=vship(counter:counter+51,1);
    DTFP5(:,i)=dtfp5(counter:counter+51,1);
    PSHIP(:,i)=piship(counter:counter+51,1);
    RVADD(:,i)=vadd(counter:counter+51,1)./piship(counter:counter+51,1);
    RSHIP(:,i)=vship(counter:counter+51,1)./piship(counter:counter+51,1);
end

% Drop asbestos sector as series is discontinued halfway through
[index1,index2]=find(sic==3292);
asbestos_index=(index1(1)-1)./52 +1;
VADD(:,asbestos_index)=[];
VSHIP(:,asbestos_index)=[];
PSHIP(:,asbestos_index)=[];
RVADD(:,asbestos_index)=[];
RSHIP(:,asbestos_index)=[];
DTFP5(:,asbestos_index)=[];

% Compute log growth rates of Real Value Added per Sector (and real value
% of shipments)
clear index1
index1=isnan(VADD);

for i=2:size(VADD,1)
    for j=1:size(VADD,2)
        if index1(i,j)==0
          DL_RVADD(i,j)=log(RVADD(i,j))-log(RVADD(i-1,j));
          DL_RSHIP(i,j)=log(RSHIP(i,j))-log(RSHIP(i-1,j)); 
        else
          DL_RVADD(i,j)=0;
          DL_RSHIP(i,j)=0;
        end
    end
end

% Form weights for aggregation
for i=1:size(VADD,1)
    for j=1:size(VADD,2)
        if index1(i,j)==0
            VADD_weight(i,j)=VADD(i,j)./nansum(VADD(i,:));
            VSHIP_weight(i,j)=VSHIP(i,j)./nansum(VSHIP(i,:));
        else
            VADD_weight(i,j)=0;
            VSHIP_weight(i,j)=0;
        end
    end
end

% Compute Aggregate Growth (aggregate real value added growth, i.e. GDP
% growth and aggregate real sales growth)
for i=1:size(VADD,1)
    agg_DLRVADD(i)=VADD_weight(i,:)*DL_RVADD(i,:)';
    agg_DLRVSHIP(i)=VSHIP_weight(i,:)*DL_RSHIP(i,:)';
end


%% IMPORT 1987 INPUT-OUTPUT DATA
load IO1987_data
W=IO;
alpha=0.5; %labor share 

%% CENTRALITY AND COMOVEMENT

% Centrality Computations
centrality=inv((eye(size(W))-0.5*W'));
v=centrality*ones(size(W,1),1); % Proportional to v vector in paper (up to constant; immaterial for computations below)
[sort_v,index]=sort(v,1,'ascend');

% Define Top 10 most central sectors
top10id=index(end-10:end);


% Compute Correlations of Sector Growth Rates with Aggregate Growth
for i=1:size(W,1)
    corr_sector_avg(i)=corr(DL_RVADD(2:end,i),agg_DLRVADD(2:end)');
    corr_sector_avg2(i)=corr(DL_RSHIP(2:end,i),agg_DLRVSHIP(2:end)');
    corr_sector_avg_tfp(i)=corr(DTFP5(2:end,i),agg_DLRVADD(2:end)');
end

% Compute Simple Average of TFP Growth in Top 10 Most Central Sectors

top10g_tfp=zeros(size(VADD,1),1);

for j=1:size(VADD,1)
    for i=1:size(top10id,1)
        top10g_tfp(j,1)=top10g_tfp(j,1)+0.1*DTFP5(j,top10id(i));
    end
end

% Figure 6

plot(1958:2009,agg_DLRVADD,1958:2009,top10g_tfp,'-o')
axis([1958 2009 -0.2 0.2])
grid on


% Correlation between tfp growth index in top 10 central sectors and
% aggregate GDP (statistics quoted in text)

corr(top10g_tfp(2:end),agg_DLRVADD(2:end)')
nanmean(corr_sector_avg_tfp) % average correlation

% Jackknife aggregate growth: 
% zeroing out weights/contribution of top 10 sectors to aggregate growth
top10id_jack=top10id;
VADD_weight_jack=VADD_weight;
VADD_weight_jack(:,top10id_jack)=0;

for i=1:size(VADD,1)
    agg_DLRVADD_jack(i)=VADD_weight_jack(i,:)*DL_RVADD(i,:)'; % jacknifed GDP growth
end

% Recompute Correlation with counterfactual GDP growth series
corr(top10g_tfp(2:end),agg_DLRVADD_jack(2:end)')




%% DISTANCE AND COMOVEMENT
% Preliminaries: Make symmetric binary matrix out of Input-Output data in
% order to compute distance (recall correlation is symmetric so we will
% have an undirected graph here, with symmetric distances as well)

IObin=zeros(max(size(IO),max(size(IO))));
for i=1:max(size(IO))
    for j=1:max(size(IO))
        if IO(i,j)>0.05
            IObin(i,j)=1;
        end
    end
end

IObinsym=(IObin+IObin');
for i=1:max(size(IObinsym))
    for j=1:max(size(IObinsym))
        if IObin(i,j)>2 
            IObinsym(i,j)=1;
        end
    end
end

% Calculate Distances across any pair of sectors

[D]=distance_bin(IObinsym');

% Calculate Correlation of Sector Output Growth
[corr_sector_growth,corr_s_pval]=corr(DL_RSHIP);


% Figure 5
% Use lower triangular part of matrix (since correlation and distance matrices is
% symmetric)
column_corr=reshape(tril(corr_sector_growth,(-1)),[],1);
[temp1,temp2]=find(column_corr==0);
column_corr(temp1)=[];

column_D=reshape(tril(D',(-1)),[],1);
column_D(temp1)=[];
clear temp1 temp2;

% Remove sectors that are not connected (infinite distance pairs)
[temp1,temp2]=find(column_D==Inf);
column_corr(temp1)=[];
column_D(temp1)=[];


% Calculate average correlation at a given distance
column_corr_per_D=zeros(size(column_corr));

for i=1:max(column_D)
    temp_index=find(column_D==i);
       column_corr_per_D(temp_index)=mean(column_corr(temp_index));
end

collapse_column_D_corr = unique([column_D column_corr_per_D], 'rows');

plot(collapse_column_D_corr(:,1),collapse_column_D_corr(:,2),'-o')
grid on
xlabel('Network Distance')
ylabel('Correlation of Sectoral Output Growth')


